@@ -1,5 +1,7 @@
-{-# LANGUAGE BangPatterns #-}
-{-# LANGUAGE RankNTypes   #-}
+{-# LANGUAGE BangPatterns             #-}
+{-# LANGUAGE CPP                      #-}
+{-# LANGUAGE ForeignFunctionInterface #-}
+{-# LANGUAGE RankNTypes               #-}
 
 module Data.HashTable.Test.Common
   ( FixedTableType
@@ -9,7 +11,7 @@ module Data.HashTable.Test.Common
   ) where
 
 ------------------------------------------------------------------------------
-import           Control.Monad                        (liftM, when)
+import           Control.Monad                        (foldM_, liftM, when)
 import           Control.Monad.ST                     (unsafeIOToST)
 import           Data.IORef
 import           Data.List                            hiding ( insert
@@ -20,14 +22,22 @@ import qualified Data.Vector                          as V
 import qualified Data.Vector.Mutable                  as MV
 import           Prelude                              hiding (lookup, mapM_)
 import           System.Random.MWC
+import           System.Timeout
 import           Test.Framework
+import           Test.Framework.Providers.HUnit
 import           Test.Framework.Providers.QuickCheck2
+import           Test.HUnit (assertFailure)
 import           Test.QuickCheck
 import           Test.QuickCheck.Monadic
 ------------------------------------------------------------------------------
 import qualified Data.HashTable.Class                 as C
 import           Data.HashTable.IO
 
+#ifndef PORTABLE
+import           Control.Concurrent
+import           Foreign (malloc, free, poke, Ptr)
+import           Foreign.C.Types (CInt)
+#endif
 
 ------------------------------------------------------------------------------
 type FixedTableType h = forall k v . IOHashTable h k v
@@ -67,6 +77,8 @@ tests prefix dummyArg = testGroup prefix $ map f ts
          , SomeTest testNewAndInsert
          , SomeTest testGrowTable
          , SomeTest testDelete
+         , SomeTest testNastyFullLookup
+         , SomeTest testForwardSearch3
          ]
 
 
@@ -252,6 +264,162 @@ testDelete prefix dummyArg =
         forceType dummyArg ht
 
 
+------------------------------------------------------------------------------
+data Action = Lookup Int
+            | Insert Int
+            | Delete Int
+            deriving Show
+
+
+timeout_ :: Int -> IO a -> IO ()
+#ifdef PORTABLE
+timeout_ t m = timeout t m >>= maybe (assertFailure "timeout")
+                                     (const $ return ())
+#else
+
+foreign import ccall safe "suicide"
+  c_suicide :: Ptr CInt -> CInt -> IO ()
+
+
+-- Foreign thread can get blocked here, stalling progress. We'll make damned
+-- sure we bomb out.
+timeout_ t m = do
+    ptr <- malloc
+    poke ptr 1
+    forkOS $ suicide ptr
+    threadDelay 1000
+    r <- timeout t m
+    poke ptr 0
+    maybe (assertFailure "timeout")
+          (const $ return ())
+          r
+  where
+    suicide ptr = do
+        c_suicide ptr $ toEnum t
+        free ptr
+#endif
+
+applyAction :: forall h . C.HashTable h =>
+               IOHashTable h Int () -> Action -> IO ()
+applyAction tbl (Lookup key) = lookup tbl key >> return ()
+applyAction tbl (Insert key) = insert tbl key ()
+applyAction tbl (Delete key) = delete tbl key
+
+
+testForwardSearch3 :: HashTest
+testForwardSearch3 prefix dummyArg = testCase (prefix ++ "/forwardSearch3") go
+  where
+    go = do
+        tbl <- new
+        forceType tbl dummyArg
+        timeout_ 3000000 $
+            foldM_ (\t k -> applyAction t k >> return t) tbl testData
+
+    testData =
+      [ Insert 65
+      , Insert 66
+      , Insert 67
+      , Insert 74
+      , Insert 75
+      , Insert 76
+      , Insert 77
+      , Insert 79
+      , Insert 80
+      , Insert 81
+      , Insert 82
+      , Insert 83
+      , Insert 84
+      , Delete 81
+      , Delete 82
+      , Insert 85
+      , Insert 86
+      , Insert 87
+      , Insert 88
+      , Insert 89
+      , Insert 90
+      , Insert 78
+      , Insert 93
+      , Insert 94
+      , Insert 95
+      , Insert 96
+      , Insert 97
+      , Insert 92
+      , Delete 93
+      , Delete 94
+      , Delete 95
+      , Delete 96
+      , Insert 99
+      , Insert 100
+      , Insert 101
+      , Insert 102
+      , Insert 103
+      , Insert 104
+      , Insert 98
+      , Insert 91
+      , Insert 108
+      , Insert 109
+      , Insert 110
+      , Insert 111
+      ]
+
+
+testNastyFullLookup :: HashTest
+testNastyFullLookup prefix dummyArg = testCase (prefix ++ "/nastyFullLookup") go
+  where
+    go = do
+        tbl <- new
+        forceType tbl dummyArg
+        timeout_ 3000000 $
+            foldM_ (\t k -> applyAction t k >> return t) tbl testData
+
+    testData =
+      [ Insert 28
+      , Insert 27
+      , Insert 30
+      , Insert 31
+      , Insert 32
+      , Insert 33
+      , Insert 34
+      , Insert 29
+      , Insert 36
+      , Insert 37
+      , Delete 34
+      , Delete 29
+      , Insert 38
+      , Insert 39
+      , Insert 40
+      , Insert 35
+      , Delete 39
+      , Insert 42
+      , Insert 43
+      , Delete 40
+      , Delete 35
+      , Insert 44
+      , Insert 45
+      , Insert 41
+      , Insert 48
+      , Insert 47
+      , Insert 50
+      , Insert 51
+      , Insert 52
+      , Insert 49
+      , Insert 54
+      , Insert 53
+      , Insert 56
+      , Insert 55
+      , Insert 58
+      , Insert 57
+      , Insert 60
+      , Insert 59
+      , Delete 60
+      , Insert 62
+      , Insert 61
+      , Insert 63
+      , Insert 46
+      , Lookup 66
+      ]
+
+
 ------------------------------------------------------------------------------
 initializeRNG :: PropertyM IO GenIO
 initializeRNG = run $ withSystemRandom (return :: GenIO -> IO GenIO)
